﻿using NAudio.Wave;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GS.Terminal.Speak
{
    public class NAudioPlayer
    {
        private const string wavDirectory = "Wav";
        private Dictionary<string, Player> wavPlayer = new Dictionary<string, Player>();
        public NAudioPlayer()
        {
            FindWav();
        }


        private void FindWav()
        {
            var newLocation = $"{Activator.AddContext.Addon.Location}\\{wavDirectory}";
            var oldLocation = $"{Activator.AddContext.Addon.Location}";
            if (!Directory.Exists(newLocation))
                Directory.CreateDirectory(newLocation);
            /*
             * 之前wav文件直接放置在插件目录下，现在转移到wav文件夹下面
             */
            var oldFiles = Directory.GetFiles(oldLocation, "*.wav", SearchOption.TopDirectoryOnly);
            foreach (var oldfile in oldFiles)
            {
                File.Copy(oldfile, $"{newLocation}\\{Path.GetFileName(oldfile)}", true);
                File.Delete(oldfile);
            }
            /*
             * 查找并初始化播放器
             */
            var wavFiles = Directory.GetFiles(newLocation, "*.wav", SearchOption.TopDirectoryOnly);
            foreach (var wavfile in wavFiles)
            {
                var msg = Path.GetFileNameWithoutExtension(wavfile);
                var player = new Player(wavfile);
                wavPlayer.Add(msg, player);
            }
        }


        public void Play(string msg)
        {
            if (!string.IsNullOrEmpty(msg))
                if (wavPlayer.TryGetValue(msg, out Player player))
                {
                    player.Play();
                }
        }
    }

    public class Player
    {
        public Player(string wavfile)
        {
            WavFile = wavfile;
            PlayerOut = new WaveOut();
            PlayerOut.PlaybackStopped += PlayerOut_PlaybackStopped;
            Reader = new AudioFileReader(wavfile);
            PlayerOut.Init(Reader);
        }

        private void PlayerOut_PlaybackStopped(object sender, StoppedEventArgs e)
        {
            if (Reader != null)
                Reader.Position = 0;
        }
        public string WavFile { get; private set; }
        public WaveOut PlayerOut { get; private set; }
        public AudioFileReader Reader { get; private set; }

        public void Play()
        {
            try
            {

                PlayerOut.Play();
            }
            catch (Exception ex)
            {
                Activator.AddContext.Logger.Error($"播放音频文件{WavFile}异常", ex);
            }
        }
    }
}
